@@ -1,22 +1,22 @@
 """
 Frame is the Client Framework that verifies and
-installs entries, and generates statistics
+installs entries, and generates statistics.
 """
-__revision__ = '$Revision: 5499 $'
+__revision__ = '$Revision$'
 
 import logging
 import time
 import Bcfg2.Client.Tools
 
 def cmpent(ent1, ent2):
-    '''Sort entries'''
+    """Sort entries."""
     if ent1.tag != ent2.tag:
         return cmp(ent1.tag, ent2.tag)
     else:
         return cmp(ent1.get('name'), ent2.get('name'))
 
 def promptFilter(prompt, entries):
-    '''Filter a supplied list based on user input'''
+    """Filter a supplied list based on user input."""
     ret = []
     entries.sort(cmpent)
     for entry in entries[:]:
@@ -27,23 +27,39 @@ def promptFilter(prompt, entries):
         try:
             if raw_input(iprompt) in ['y', 'Y']:
                 ret.append(entry)
+        except EOFError:
+            # python 2.4.3 on CentOS doesn't like ^C for some reason
+            break
         except:
             print("Error while reading input")
             continue
     return ret
 
+def matches_entry(entryspec, entry):
+    # both are (tag, name)
+    if entryspec == entry:
+        return True
+    else:
+        for i in [0, 1]:
+            if entryspec[i] == entry[i]:
+                continue
+            elif entryspec[i] == '*':
+                continue
+            elif '*' in entryspec[i]:
+                starpt = entryspec[i].index('*')
+                if entry[i].startswith(entryspec[i][:starpt]):
+                    continue
+            return False
+        return True
+
 def matches_white_list(entry, whitelist):
-    return [entry.tag, entry.get('name')] in whitelist or \
-           [entry.tag, '*'] in whitelist or \
-           ['*', entry.get('name')] in whitelist
+    return True in [matches_entry(we, (entry.tag, entry.get('name'))) for we in whitelist]
 
 def passes_black_list(entry, blacklist):
-    return [entry.tag, entry.get('name')] not in blacklist and \
-           [entry.tag, '*'] not in blacklist and \
-           ['*', entry.get('name')] not in blacklist
+    return True not in [matches_entry(be, (entry.tag, entry.get('name'))) for be in blacklist]
 
 class Frame:
-    '''Frame is the container for all Tool objects and state information'''
+    """Frame is the container for all Tool objects and state information."""
     def __init__(self, config, setup, times, drivers, dryrun):
         self.config = config
         self.times = times
@@ -108,6 +124,21 @@ class Frame:
                         except:
                             self.logger.error("Unexpected tool failure",
                                               exc_info=1)
+            for cfile in [cfl for cfl in config.findall(".//Path") \
+                          if cfl.get('name') in self.__important__ and \
+                             cfl.get('type') == 'file']:
+                tl = [t for t in self.tools if t.handlesEntry(cfile) \
+                     and t.canVerify(cfile)]
+                if tl:
+                    if not tl[0].VerifyPath(cfile, []):
+                        if self.setup['interactive'] and not \
+                               promptFilter("Install %s: %s? (y/N):", [cfile]):
+                            continue
+                        try:
+                            self.states[cfile] = tl[0].InstallPath(cfile)
+                        except:
+                            self.logger.error("Unexpected tool failure",
+                                              exc_info=1)
         # find entries not handled by any tools
         problems = [entry for struct in config for \
                     entry in struct if entry not in self.handled]
@@ -146,11 +177,12 @@ class Frame:
         raise AttributeError(name)
 
     def Inventory(self):
-        '''
+        """
            Verify all entries,
            find extra entries,
            and build up workqueues
-        '''
+
+        """
         # initialize all states
         for struct in self.config.getchildren():
             for entry in struct.getchildren():
@@ -162,7 +194,7 @@ class Frame:
                 self.logger.error("%s.Inventory() call failed:" % tool.name, exc_info=1)
 
     def Decide(self):
-        '''Set self.whitelist based on user interaction'''
+        """Set self.whitelist based on user interaction."""
         prompt = "Install %s: %s? (y/N): "
         rprompt = "Remove %s: %s? (y/N): "
         if self.setup['remove']:
@@ -262,7 +294,7 @@ class Frame:
                 self.blacklist.append(entry)
 
     def DispatchInstallCalls(self, entries):
-        '''Dispatch install calls to underlying tools'''
+        """Dispatch install calls to underlying tools."""
         for tool in self.tools:
             handled = [entry for entry in entries if tool.canInstall(entry)]
             if not handled:
@@ -273,7 +305,7 @@ class Frame:
                 self.logger.error("%s.Install() call failed:" % tool.name, exc_info=1)
 
     def Install(self):
-        '''Install all entries'''
+        """Install all entries."""
         self.DispatchInstallCalls(self.whitelist)
         mods = self.modified
         mbundles = [struct for struct in self.config.findall('Bundle') if \
@@ -316,7 +348,7 @@ class Frame:
                                       (tool.name), exc_info=1)
 
     def Remove(self):
-        '''Remove extra entries'''
+        """Remove extra entries."""
         for tool in self.tools:
             extras = [entry for entry in self.removal if tool.handlesEntry(entry)]
             if extras:
@@ -326,7 +358,7 @@ class Frame:
                     self.logger.error("%s.Remove() failed" % tool.name, exc_info=1)
 
     def CondDisplayState(self, phase):
-        '''Conditionally print tracing information'''
+        """Conditionally print tracing information."""
         self.logger.info('\nPhase: %s' % phase)
         self.logger.info('Correct entries:\t%d' % list(self.states.values()).count(True))
         self.logger.info('Incorrect entries:\t%d' % list(self.states.values()).count(False))
@@ -345,13 +377,13 @@ class Frame:
             self.logger.info('All entries correct.')
 
     def ReInventory(self):
-        '''Recheck everything'''
+        """Recheck everything."""
         if not self.dryrun and self.setup['kevlar']:
             self.logger.info("Rechecking system inventory")
             self.Inventory()
 
     def Execute(self):
-        '''Run all methods'''
+        """Run all methods."""
         self.Inventory()
         self.times['inventory'] = time.time()
         self.CondDisplayState('initial')
@@ -367,7 +399,7 @@ class Frame:
         self.CondDisplayState('final')
 
     def GenerateStats(self):
-        '''Generate XML summary of execution statistics'''
+        """Generate XML summary of execution statistics."""
         feedback = Bcfg2.Client.XML.Element("upload-statistics")
         stats = Bcfg2.Client.XML.SubElement(feedback,
                                             'Statistics',
