@@ -1,5 +1,5 @@
-'''This is the bcfg2 support for apt-get'''
-__revision__ = '$Revision: 5546 $'
+"""This is the bcfg2 support for apt-get"""
+__revision__ = '$Revision$'
 
 # suppress apt API warnings
 import warnings
@@ -7,44 +7,69 @@ warnings.filterwarnings("ignore", "apt API not stable yet",
                         FutureWarning)
 warnings.filterwarnings("ignore", "Accessed deprecated property Package.installedVersion, please see the Version class for alternatives.", DeprecationWarning)
 warnings.filterwarnings("ignore", "Accessed deprecated property Package.candidateVersion, please see the Version class for alternatives.", DeprecationWarning)
+warnings.filterwarnings("ignore", "Deprecated, please use 'is_installed' instead", DeprecationWarning)
+warnings.filterwarnings("ignore", "Attribute 'IsUpgradable' of the 'apt_pkg.DepCache' object is deprecated, use 'is_upgradable' instead.", DeprecationWarning)
 import apt.cache
 import os
+
 import Bcfg2.Client.Tools
+import Bcfg2.Options
+
+# Options for tool locations
+opts = {'install_path': Bcfg2.Options.CLIENT_APT_TOOLS_INSTALL_PATH,
+        'var_path': Bcfg2.Options.CLIENT_APT_TOOLS_VAR_PATH,
+        'etc_path': Bcfg2.Options.CLIENT_SYSTEM_ETC_PATH}
+setup = Bcfg2.Options.OptionParser(opts)
+setup.parse([])
+install_path = setup['install_path']
+var_path = setup['var_path']
+etc_path = setup['etc_path']
+DEBSUMS = '%s/bin/debsums' % install_path
+APTGET = '%s/bin/apt-get' % install_path
+DPKG = '%s/bin/dpkg' % install_path
 
 class APT(Bcfg2.Client.Tools.Tool):
-    '''The Debian toolset implements package and service operations and inherits
-    the rest from Toolset.Toolset'''
+    """The Debian toolset implements package and service operations and inherits
+    the rest from Toolset.Toolset.
+
+    """
     name = 'APT'
-    __execs__ = ['/usr/bin/debsums', '/usr/bin/apt-get', '/usr/bin/dpkg']
-    __important__ = ["/etc/apt/sources.list",
-                     "/var/cache/debconf/config.dat",
-                     "/var/cache/debconf/templates.dat",
-                     '/etc/passwd', '/etc/group',
-                     '/etc/apt/apt.conf', '/etc/dpkg/dpkg.cfg']
+    __execs__ = [DEBSUMS, APTGET, DPKG]
     __handles__ = [('Package', 'deb')]
     __req__ = {'Package': ['name', 'version']}
-    pkgcmd = 'apt-get ' + \
+    pkgcmd = '%s ' % APTGET + \
              '-o DPkg::Options::=--force-overwrite ' + \
              '-o DPkg::Options::=--force-confold ' + \
+             '-o DPkg::Options::=--force-confmiss ' + \
              '--reinstall ' + \
              '-q=2 ' + \
              '--force-yes ' + \
              '-y install %s'
 
-    def __init__(self, logger, cfg, setup):
-        Bcfg2.Client.Tools.Tool.__init__(self, logger, cfg, setup)
-        self.cfg = cfg
+    def __init__(self, logger, setup, config):
+        Bcfg2.Client.Tools.Tool.__init__(self, logger, setup, config)
+        self.__important__ = self.__important__ + \
+                             ["%s/cache/debconf/config.dat" % var_path,
+                              "%s/cache/debconf/templates.dat" % var_path,
+                              '/etc/passwd', '/etc/group',
+                              '%s/apt/apt.conf' % etc_path,
+                              '%s/dpkg/dpkg.cfg' % etc_path] + \
+                             [entry.get('name') for struct in config for entry in struct \
+                              if entry.tag in ['Path', 'ConfigFile'] and \
+                              entry.get('name').startswith('%s/apt/sources.list' % etc_path)]
+        self.nonexistent = [entry.get('name') for struct in config for entry in struct \
+                              if entry.tag == 'Path' and entry.get('type') == 'nonexistent']
         os.environ["DEBIAN_FRONTEND"] = 'noninteractive'
         self.actions = {}
         if self.setup['kevlar'] and not self.setup['dryrun']:
-            self.cmd.run("dpkg --force-confold --configure --pending")
-            self.cmd.run("apt-get clean")
+            self.cmd.run("%s --force-confold --configure --pending" % DPKG)
+            self.cmd.run("%s clean" % APTGET)
             self.pkg_cache = apt.cache.Cache()
             self.pkg_cache.update()
         self.pkg_cache = apt.cache.Cache()
 
     def FindExtra(self):
-        '''Find extra packages'''
+        """Find extra packages."""
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
         extras = [(p.name, p.installedVersion) for p in self.pkg_cache
                   if p.isInstalled and p.name not in packages]
@@ -53,7 +78,7 @@ class APT(Bcfg2.Client.Tools.Tool):
                                          for (name, version) in extras]
 
     def VerifyDebsums(self, entry, modlist):
-        output = self.cmd.run("/usr/bin/debsums -as %s" % entry.get('name'))[1]
+        output = self.cmd.run("%s -as %s" % (DEBSUMS, entry.get('name')))[1]
         if len(output) == 1 and "no md5sums for" in output[0]:
             self.logger.info("Package %s has no md5sums. Cannot verify" % \
                              entry.get('name'))
@@ -64,9 +89,16 @@ class APT(Bcfg2.Client.Tools.Tool):
         for item in output:
             if "checksum mismatch" in item:
                 files.append(item.split()[-1])
+            elif "changed file" in item:
+                files.append(item.split()[3])
             elif "can't open" in item:
-                files.append(item.split()[5])
-            elif "is not installed" in item:
+                if item.split()[5] not in self.nonexistent:
+                    files.append(item.split()[5])
+            elif "missing file" in item and \
+                 item.split()[3] in self.nonexistent:
+                # these files should not exist
+                continue
+            elif "is not installed" in item or "missing file" in item:
                 self.logger.error("Package %s is not fully installed" \
                                   % entry.get('name'))
             else:
@@ -90,7 +122,7 @@ class APT(Bcfg2.Client.Tools.Tool):
         return True
 
     def VerifyPackage(self, entry, modlist, checksums=True):
-        '''Verify package for entry'''
+        """Verify package for entry."""
         if not 'version' in entry.attrib:
             self.logger.info("Cannot verify unversioned package %s" %
                              (entry.attrib['name']))
@@ -116,7 +148,7 @@ class APT(Bcfg2.Client.Tools.Tool):
             entry.set('current_version', pkg.installedVersion)
             entry.set('qtext', "Modify Package %s (%s -> %s)? (y/N) " % \
                       (entry.get('name'), entry.get('current_version'),
-                       entry.get('version')))
+                       desiredVersion))
             return False
         else:
             # version matches
@@ -127,7 +159,7 @@ class APT(Bcfg2.Client.Tools.Tool):
             return True
 
     def Remove(self, packages):
-        '''Deal with extra configuration detected'''
+        """Deal with extra configuration detected."""
         pkgnames = " ".join([pkg.get('name') for pkg in packages])
         self.pkg_cache = apt.cache.Cache()
         if len(packages) > 0:
