@@ -1,11 +1,18 @@
-'''Debian Init Support for Bcfg2'''
-__revision__ = '$Revision: 5408 $'
+"""Debian Init Support for Bcfg2"""
+__revision__ = '$Revision$'
 
 import glob, os, re
 import Bcfg2.Client.Tools
 
+
+# Debian squeeze and beyond uses a dependecy based boot sequence
+DEBIAN_OLD_STYLE_BOOT_SEQUENCE = (
+    'etch', '4.0',
+    'lenny', '5.0', '5.0.1', '5.0.2', '5.0.3', '5.0.4', '5.0.4', '5.0.5',
+    )
+
 class DebInit(Bcfg2.Client.Tools.SvcTool):
-    '''Debian Service Support for Bcfg2'''
+    """Debian Service Support for Bcfg2."""
     name = 'DebInit'
     __execs__ = ['/usr/sbin/update-rc.d', '/usr/sbin/invoke-rc.d']
     __handles__ = [('Service', 'deb')]
@@ -14,12 +21,21 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
 
     # implement entry (Verify|Install) ops
     def VerifyService(self, entry, _):
-        '''Verify Service status for entry'''
+        """Verify Service status for entry."""
         rawfiles = glob.glob("/etc/rc*.d/[SK]*%s" % (entry.get('name')))
         files = []
+
+        try:
+            deb_version = open('/etc/debian_version', 'r').read().split('/', 1)[0]
+        except IOError:
+            deb_version = 'unknown'
+
         if entry.get('sequence'):
-            start_sequence = int(entry.get('sequence'))
-            kill_sequence = 100 - start_sequence
+            if (deb_version in DEBIAN_OLD_STYLE_BOOT_SEQUENCE):
+                start_sequence = int(entry.get('sequence'))
+                kill_sequence = 100 - start_sequence
+            else:
+                self.logger.warning("Your debian version boot sequence is dependency based \"sequence\" attribute wil be ignored.")
         else:
             start_sequence = None
 
@@ -52,7 +68,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
                 return False
 
     def InstallService(self, entry):
-        '''Install Service for entry'''
+        """Install Service for entry."""
         self.logger.info("Installing Service %s" % (entry.get('name')))
         try:
             os.stat('/etc/init.d/%s' % entry.get('name'))
@@ -76,7 +92,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
         return cmdrc == 0
 
     def FindExtra(self):
-        '''Find Extra Debian Service Entries'''
+        """Find Extra Debian Service entries."""
         specified = [entry.get('name') for entry in self.getSupportedEntries()]
         extra = []
         for name in [self.svcre.match(fname).group('name') for fname in
@@ -88,7 +104,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
                 in extra]
 
     def Remove(self, _):
-        '''Remove extra service entries'''
+        """Remove extra service entries."""
         # Extra service removal is nonsensical
         # Extra services need to be reflected in the config
         return
