@@ -1,5 +1,5 @@
-'''FreeBSD Init Support for Bcfg2'''
-__revision__ = '$Rev: 5430 $'
+"""FreeBSD Init Support for Bcfg2."""
+__revision__ = '$Rev$'
 
 # TODO
 # - hardcoded path to ports rc.d
@@ -9,12 +9,13 @@ import os
 import Bcfg2.Client.Tools
 
 class FreeBSDInit(Bcfg2.Client.Tools.SvcTool):
-    '''FreeBSD Service Support for Bcfg2'''
+    """FreeBSD service support for Bcfg2."""
     name = 'FreeBSDInit'
     __handles__ = [('Service', 'freebsd')]
     __req__ = {'Service': ['name', 'status']}
 
     def __init__(self, logger, cfg, setup):
+        Bcfg2.Client.Tools.Tool.__init__(self, logger, cfg, setup)
         if os.uname()[0] != 'FreeBSD':
             raise Bcfg2.Client.Tools.toolInstantiationError
 
