@@ -1,5 +1,5 @@
-'''Bcfg2 logging support'''
-__revision__ = '$Revision: 5477 $'
+"""Bcfg2 logging support"""
+__revision__ = '$Revision$'
 
 import copy
 import fcntl
@@ -14,16 +14,16 @@ import termios
 logging.raiseExceptions = 0
 
 def print_attributes(attrib):
-    ''' Add the attributes for an element'''
+    """Add the attributes for an element."""
     return ' '.join(['%s="%s"' % data for data in list(attrib.items())])
 
 def print_text(text):
-    ''' Add text to the output (which will need normalising '''
+    """Add text to the output (which will need normalising."""
     charmap = {'<':'&lt;', '>':'&gt;', '&':'&amp;'}
     return ''.join([charmap.get(char, char) for char in text]) + '\n'
 
 def xml_print(element, running_indent=0, indent=4):
-    ''' Add an element and its children to the return string '''
+    """Add an element and its children to the return string."""
     if (len(element.getchildren()) == 0) and (not element.text):
         ret = (' ' * running_indent)
         ret += '<%s %s/>\n' % (element.tag, print_attributes(element.attrib))
@@ -41,7 +41,7 @@ def xml_print(element, running_indent=0, indent=4):
     return ret
 
 class TermiosFormatter(logging.Formatter):
-    '''The termios formatter displays output in a terminal-sensitive fashion'''
+    """The termios formatter displays output in a terminal-sensitive fashion."""
 
     def __init__(self, fmt=None, datefmt=None):
         logging.Formatter.__init__(self, fmt, datefmt)
@@ -92,10 +92,10 @@ class TermiosFormatter(logging.Formatter):
         return '\n'.join(returns)
 
 class FragmentingSysLogHandler(logging.handlers.SysLogHandler):
-    '''
+    """
        This handler fragments messages into
        chunks smaller than 250 characters
-    '''
+    """
 
     def __init__(self, procname, path, facility):
         self.procname = procname
@@ -103,7 +103,7 @@ class FragmentingSysLogHandler(logging.handlers.SysLogHandler):
         logging.handlers.SysLogHandler.__init__(self, path, facility)
 
     def emit(self, record):
-        '''chunk and deliver records'''
+        """Chunk and deliver records."""
         record.name = self.procname
         if str(record.msg) > 250:
             msgs = []
@@ -118,8 +118,7 @@ class FragmentingSysLogHandler(logging.handlers.SysLogHandler):
             msgs[0].exc_info = error
         else:
             msgs = [record]
-        while msgs:
-            newrec = msgs.pop()
+        for newrec in msgs:
             msg = self.log_format_string % (self.encodePriority(self.facility,
                                                                 newrec.levelname.lower()), self.format(newrec))
             try:
@@ -135,11 +134,18 @@ class FragmentingSysLogHandler(logging.handlers.SysLogHandler):
                         break
                     except socket.error:
                         continue
-                self.socket.send("Reconnected to syslog")
-                self.socket.send(msg)
+                try:
+                    self.socket.send("Reconnected to syslog")
+                    self.socket.send(msg)
+                except:
+                    """
+                    If we still fail then drop it.  Running bcfg2-server as non-root can
+                    trigger permission denied exceptions.
+                    """
+                    pass
 
 def setup_logging(procname, to_console=True, to_syslog=True, syslog_facility='daemon', level=0, to_file=None):
-    '''setup logging for bcfg2 software'''
+    """Setup logging for Bcfg2 software."""
     if hasattr(logging, 'already_setup'):
         return
     # add the handler to the root logger
@@ -165,18 +171,19 @@ def setup_logging(procname, to_console=True, to_syslog=True, syslog_facility='da
     if not to_file == None:
         filelog = logging.FileHandler(to_file)
         filelog.setLevel(logging.DEBUG)
-        filelog.setFormatter(logging.Formatter('%(name)s[%(process)d]: %(message)s'))
+        filelog.setFormatter(logging.Formatter('%(asctime)s %(name)s[%(process)d]: %(message)s'))
         logging.root.addHandler(filelog)
     logging.root.setLevel(level)
     logging.already_setup = True
 
 def trace_process(**kwargs):
-
     """Literally log every line of python code as it runs.
 
     Keyword arguments:
     log -- file (name) to log to (default stderr)
-    scope -- base scope to log to (default Cobalt)"""
+    scope -- base scope to log to (default Cobalt)
+
+    """
 
     file_name = kwargs.get("log", None)
     if file_name is not None:
