@@ -1,12 +1,15 @@
-'''This module provides the baseclass for Bcfg2 Server Plugins'''
-__revision__ = '$Revision: 5651 $'
+"""This module provides the baseclass for Bcfg2 Server Plugins."""
+__revision__ = '$Revision$'
 
 import copy
 import logging
 import lxml.etree
 import os
+import pickle
 import posixpath
 import re
+import Queue
+import threading
 
 from lxml.etree import XML, XMLSyntaxError
 
@@ -15,6 +18,7 @@ import Bcfg2.Options
 # grab default metadata info from bcfg2.conf
 opts = {'owner': Bcfg2.Options.MDATA_OWNER,
         'group': Bcfg2.Options.MDATA_GROUP,
+        'important': Bcfg2.Options.MDATA_IMPORTANT,
         'perms': Bcfg2.Options.MDATA_PERMS,
         'paranoid': Bcfg2.Options.MDATA_PARANOID}
 mdata_setup = Bcfg2.Options.OptionParser(opts)
@@ -26,20 +30,24 @@ logger = logging.getLogger('Bcfg2.Plugin')
 default_file_metadata = mdata_setup
 
 info_regex = re.compile( \
-    '^owner:(\s)*(?P<owner>\S+)|group:(\s)*(?P<group>\S+)|' +
-    'perms:(\s)*(?P<perms>\w+)|encoding:(\s)*(?P<encoding>\w+)|' +
-    'paranoid:(\s)*(?P<paranoid>\S+)|mtime:(\s)*(?P<mtime>\w+)$')
+    'encoding:(\s)*(?P<encoding>\w+)|' +
+    'group:(\s)*(?P<group>\S+)|' +
+    'important:(\s)*(?P<important>\S+)|' +
+    'mtime:(\s)*(?P<mtime>\w+)|' +
+    'owner:(\s)*(?P<owner>\S+)|' +
+    'paranoid:(\s)*(?P<paranoid>\S+)|' +
+    'perms:(\s)*(?P<perms>\w+)|')
 
 class PluginInitError(Exception):
-    '''Error raised in cases of Plugin initialization errors'''
+    """Error raised in cases of Plugin initialization errors."""
     pass
 
 class PluginExecutionError(Exception):
-    '''Error raised in case of Plugin execution errors'''
+    """Error raised in case of Plugin execution errors."""
     pass
 
 class Plugin(object):
-    '''This is the base class for all Bcfg2 Server plugins.
+    """This is the base class for all Bcfg2 Server plugins.
     Several attributes must be defined in the subclass:
     name : the name of the plugin
     __version__ : a version string
@@ -49,13 +57,14 @@ class Plugin(object):
       - Structure creation (overloading BuildStructures)
       - Configuration entry binding (overloading HandlesEntry, or loads the Entries table)
       - Data collection (overloading GetProbes/ReceiveData)
-    '''
+    """
     name = 'Plugin'
-    __version__ = '$Id: Plugin.py 5651 2010-01-04 00:59:45Z solj $'
+    __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __rmi__ = ['toggle_debug']
     experimental = False
     deprecated = False
+    conflicts = []
 
     def __init__(self, core, datastore):
         object.__init__(self)
@@ -73,50 +82,40 @@ class Plugin(object):
         if (flag is None) and self.debug_flag or flag:
             self.logger.error(message)
 
-    @staticmethod
-    def make_path(path):
-        p_comp = path.split('/')
-        for i in range(2, len(p_comp) + 1):
-            ppath = '/' + '/'.join(p_comp[1:i])
-            try:
-                os.stat(ppath)
-            except:
-                os.mkdir(ppath)
-
     @classmethod
     def init_repo(cls, repo):
         path = "%s/%s" % (repo, cls.name)
-        cls.make_path(path)
+        os.makedirs(path)
 
     def shutdown(self):
         self.running = False
 
 class Generator(object):
-    '''Generator plugins contribute to literal client configurations'''
+    """Generator plugins contribute to literal client configurations."""
     def HandlesEntry(self, entry, metadata):
-        '''This is the slow path method for routing configuration binding requests'''
+        """This is the slow path method for routing configuration binding requests."""
         return False
 
     def HandleEntry(self, entry, metadata):
-        '''This is the slow-path handler for configuration entry binding'''
+        """This is the slow-path handler for configuration entry binding."""
         raise PluginExecutionError
 
 class Structure(object):
-    '''Structure Plugins contribute to abstract client configurations'''
+    """Structure Plugins contribute to abstract client configurations."""
     def BuildStructures(self, metadata):
-        '''return a list of abstract goal structures for client'''
+        """Return a list of abstract goal structures for client."""
         raise PluginExecutionError
 
 class Metadata(object):
-    '''Signal metadata capabilities for this plugin'''
+    """Signal metadata capabilities for this plugin"""
     def add_client(self, client_name, attribs):
-        '''add client'''
+        """Add client."""
         pass
     def remove_client(self, client_name):
-        '''remove client'''
+        """Remove client."""
         pass
     def viz(self, hosts, bundles, key, colors):
-        '''create viz str for viz admin mode'''
+        """Create viz str for viz admin mode."""
         pass
 
     def get_initial_metadata(self, client_name):
@@ -126,30 +125,136 @@ class Metadata(object):
         raise PluginExecutionError
 
 class Connector(object):
-    '''Connector Plugins augment client metadata instances'''
+    """Connector Plugins augment client metadata instances."""
     def get_additional_groups(self, metadata):
-        '''determine additional groups for metadata'''
+        """Determine additional groups for metadata."""
         return list()
 
     def get_additional_data(self, metadata):
-        '''determine additional data for metadata inst'''
+        """Determine additional data for metadata instances."""
         return dict()
 
 class Probing(object):
-    '''Signal probe capability for this plugin'''
+    """Signal probe capability for this plugin."""
     def GetProbes(self, _):
-        '''Return a set of probes for execution on client'''
+        """Return a set of probes for execution on client."""
         return []
 
     def ReceiveData(self, _, dummy):
-        '''Receive probe results pertaining to client'''
+        """Receive probe results pertaining to client."""
         pass
 
 class Statistics(object):
-    '''Signal statistics handling capability'''
+    """Signal statistics handling capability."""
     def process_statistics(self, client, xdata):
         pass
 
+class ThreadedStatistics(Statistics,
+                         threading.Thread):
+    """Threaded statistics handling capability."""
+    def __init__(self, core, datastore):
+        Statistics.__init__(self)
+        threading.Thread.__init__(self)
+        # Event from the core signaling an exit
+        self.terminate = core.terminate
+        self.work_queue = Queue.Queue(100000)
+        self.pending_file = "%s/etc/%s.pending" % (datastore, self.__class__.__name__)
+        self.daemon = True
+        self.start()
+
+    def save(self):
+        """Save any pending data to a file."""
+        pending_data = []
+        try:
+            while not self.work_queue.empty():
+                (metadata, data) = self.work_queue.get_nowait()
+                try:
+                    pending_data.append( ( metadata.hostname, lxml.etree.tostring(data) ) )
+                except:
+                    self.logger.warning("Dropping interaction for %s" % metadata.hostname)
+        except Queue.Empty:
+            pass
+
+        try:
+            savefile = open(self.pending_file, 'w')
+            pickle.dump(pending_data, savefile)
+            savefile.close()
+            self.logger.info("Saved pending %s data" % self.__class__.__name__)
+        except:
+            self.logger.warning("Failed to save pending data")
+
+    def load(self):
+        """Load any pending data to a file."""
+        if not os.path.exists(self.pending_file):
+            return True
+        pending_data = []
+        try:
+            savefile = open(self.pending_file, 'r')
+            pending_data = pickle.load(savefile)
+            savefile.close()
+        except Exception, e:
+            self.logger.warning("Failed to load pending data: %s" % e)
+        for (pmetadata, pdata) in pending_data:
+            # check that shutdown wasnt called early
+            if self.terminate.isSet():
+                return False
+
+            try:
+                while True:
+                    try:
+		        metadata = self.core.build_metadata(pmetadata)
+                        break
+                    except Bcfg2.Server.Plugins.Metadata.MetadataRuntimeError:
+                        pass
+
+                    self.terminate.wait(5)
+                    if self.terminate.isSet():
+                        return False
+
+                self.work_queue.put_nowait( (metadata, lxml.etree.fromstring(pdata)) )
+            except Queue.Full:
+                self.logger.warning("Queue.Full: Failed to load queue data")
+                break
+            except lxml.etree.LxmlError, lxml_error:
+                self.logger.error("Unable to load save interaction: %s" % lxml_error)
+            except Bcfg2.Server.Plugins.Metadata.MetadataConsistencyError:
+                self.logger.error("Unable to load metadata for save interaction: %s" % pmetadata)
+        try:
+            os.unlink(self.pending_file)
+        except:
+            self.logger.error("Failed to unlink save file: %s" % self.pending_file)
+        self.logger.info("Loaded pending %s data" % self.__class__.__name__)
+        return True
+
+    def run(self):
+        if not self.load():
+            return
+        while not self.terminate.isSet():
+            try:
+                (xdata, client) = self.work_queue.get(block=True, timeout=2)
+            except Queue.Empty:
+                continue
+            except Exception, e:
+                self.logger.error("ThreadedStatistics: %s" % e)
+                continue
+            self.handle_statistic(xdata, client)
+        if not self.work_queue.empty():
+            self.save()
+
+    def process_statistics(self, metadata, data):
+        warned = False
+        try:
+            self.work_queue.put_nowait((metadata, copy.deepcopy(data)))
+            warned = False
+        except Queue.Full:
+            if not warned:
+                self.logger.warning("%s: Queue is full.  Dropping interactions." % self.__class__.__name__)
+            warned = True
+
+    def handle_statistics(self, metadata, data):
+        """Handle stats here."""
+        pass
+
 class PullSource(object):
     def GetExtra(self, client):
         return []
@@ -162,12 +267,12 @@ class PullTarget(object):
         raise PluginExecutionError
 
     def AcceptPullData(self, specific, new_entry, verbose):
-        '''This is the null per-plugin implementation
-        of bcfg2-admin pull'''
+        """This is the null per-plugin implementation
+        of bcfg2-admin pull."""
         raise PluginExecutionError
 
 class Decision(object):
-    '''Signal decision handling capability'''
+    """Signal decision handling capability."""
     def GetDecisions(self, metadata, mode):
         return []
 
@@ -175,17 +280,17 @@ class ValidationError(Exception):
     pass
 
 class StructureValidator(object):
-    '''Validate/modify goal structures'''
+    """Validate/modify goal structures."""
     def validate_structures(self, metadata, structures):
         raise ValidationError, "not implemented"
 
 class GoalValidator(object):
-    '''Validate/modify configuration goals'''
+    """Validate/modify configuration goals."""
     def validate_goals(self, metadata, goals):
         raise ValidationError, "not implemented"
 
 class Version(object):
-    '''Interact with various version control systems'''
+    """Interact with various version control systems."""
     def get_revision(self):
         return []
     def commit_data(self):
@@ -194,10 +299,11 @@ class Version(object):
 # the rest of the file contains classes for coherent file caching
 
 class FileBacked(object):
-    '''This object caches file data in memory.
+    """This object caches file data in memory.
     HandleEvent is called whenever fam registers an event.
     Index can parse the data into member data as required.
-    This object is meant to be used as a part of DirectoryBacked.'''
+    This object is meant to be used as a part of DirectoryBacked.
+    """
 
     def __init__(self, name):
         object.__init__(self)
@@ -205,7 +311,7 @@ class FileBacked(object):
         self.name = name
 
     def HandleEvent(self, event=None):
-        '''Read file upon update'''
+        """Read file upon update."""
         if event and event.code2str() not in ['exists', 'changed', 'created']:
             return
         try:
@@ -215,11 +321,11 @@ class FileBacked(object):
             logger.error("Failed to read file %s" % (self.name))
 
     def Index(self):
-        '''Update local data structures based on current file state'''
+        """Update local data structures based on current file state"""
         pass
 
 class DirectoryBacked(object):
-    '''This object is a coherent cache for a filesystem hierarchy of files.'''
+    """This object is a coherent cache for a filesystem hierarchy of files."""
     __child__ = FileBacked
     patterns = re.compile('.*')
 
@@ -238,7 +344,7 @@ class DirectoryBacked(object):
         return self.entries.iteritems()
 
     def AddEntry(self, name):
-        '''Add new entry to data structures upon file creation'''
+        """Add new entry to data structures upon file creation."""
         if name == '':
             logger.info("got add for empty name")
         elif name in self.entries:
@@ -255,7 +361,7 @@ class DirectoryBacked(object):
             self.entries[name].HandleEvent()
 
     def HandleEvent(self, event):
-        '''Propagate fam events to underlying objects'''
+        """Propagate fam events to underlying objects."""
         action = event.code2str()
         if event.filename == '':
             logger.info("Got event for blank filename")
@@ -279,7 +385,10 @@ class DirectoryBacked(object):
                                                   event.filename)
 
 class XMLFileBacked(FileBacked):
-    '''This object is a coherent cache for an XML file to be used as a part of DirectoryBacked.'''
+    """
+    This object is a coherent cache for an XML file to be used as a
+    part of DirectoryBacked.
+    """
     __identifier__ = 'name'
 
     def __init__(self, filename):
@@ -288,7 +397,7 @@ class XMLFileBacked(FileBacked):
         FileBacked.__init__(self, filename)
 
     def Index(self):
-        '''Build local data structures'''
+        """Build local data structures."""
         try:
             xdata = XML(self.data)
         except XMLSyntaxError:
@@ -301,19 +410,19 @@ class XMLFileBacked(FileBacked):
         return iter(self.entries)
 
 class SingleXMLFileBacked(XMLFileBacked):
-    '''This object is a coherent cache for an independent XML File.'''
+    """This object is a coherent cache for an independent XML file."""
     def __init__(self, filename, fam):
         XMLFileBacked.__init__(self, filename)
         fam.AddMonitor(filename, self)
 
 class StructFile(XMLFileBacked):
-    '''This file contains a set of structure file formatting logic'''
+    """This file contains a set of structure file formatting logic."""
     def __init__(self, name):
         XMLFileBacked.__init__(self, name)
         self.fragments = {}
 
     def Index(self):
-        '''Build internal data structures'''
+        """Build internal data structures."""
         try:
             xdata = lxml.etree.XML(self.data)
         except lxml.etree.XMLSyntaxError:
@@ -336,7 +445,7 @@ class StructFile(XMLFileBacked):
                 work[newpred] = group.getchildren()
 
     def Match(self, metadata):
-        '''Return matching fragments of independent'''
+        """Return matching fragments of independent."""
         matching = [frag for (pred, frag) in self.fragments.iteritems() if pred(metadata)]
         if matching:
             return reduce(lambda x, y:x+y, matching)
@@ -344,7 +453,10 @@ class StructFile(XMLFileBacked):
         return []
 
 class INode:
-    '''LNodes provide lists of things available at a particular group intersection'''
+    """
+    LNodes provide lists of things available at a particular
+    group intersection.
+    """
     raw = {'Client':"lambda x:'%s' == x.hostname and predicate(x)",
            'Group':"lambda x:'%s' in x.groups and predicate(x)"}
     nraw = {'Client':"lambda x:'%s' != x.hostname and predicate(x)",
@@ -388,7 +500,7 @@ class INode:
                     idict[item.tag] = [item.get('name')]
 
     def Match(self, metadata, data):
-        '''Return a dictionary of package mappings'''
+        """Return a dictionary of package mappings."""
         if self.predicate(metadata):
             for key in self.contents:
                 try:
@@ -400,7 +512,7 @@ class INode:
                 child.Match(metadata, data)
 
 class XMLSrc(XMLFileBacked):
-    '''XMLSrc files contain a LNode hierarchy that returns matching entries'''
+    """XMLSrc files contain a LNode hierarchy that returns matching entries."""
     __node__ = INode
     __cacheobj__ = dict
 
@@ -413,7 +525,7 @@ class XMLSrc(XMLFileBacked):
         self.noprio = noprio
 
     def HandleEvent(self, _=None):
-        '''Read file upon update'''
+        """Read file upon update."""
         try:
             data = file(self.name).read()
         except IOError:
@@ -435,7 +547,7 @@ class XMLSrc(XMLFileBacked):
         del xdata, data
 
     def Cache(self, metadata):
-        '''Build a package dict for a given host'''
+        """Build a package dict for a given host."""
         if self.cache == None or self.cache[0] != metadata:
             cache = (metadata, self.__cacheobj__())
             if self.pnode == None:
@@ -446,11 +558,11 @@ class XMLSrc(XMLFileBacked):
             self.cache = cache
 
 class XMLDirectoryBacked(DirectoryBacked):
-    '''Directorybacked for *.xml'''
+    """Directorybacked for *.xml."""
     patterns = re.compile('.*\.xml')
 
 class PrioDir(Plugin, Generator, XMLDirectoryBacked):
-    '''This is a generator that handles package assignments'''
+    """This is a generator that handles package assignments."""
     name = 'PrioDir'
     __child__ = XMLSrc
 
@@ -464,7 +576,7 @@ class PrioDir(Plugin, Generator, XMLDirectoryBacked):
             raise PluginInitError
 
     def HandleEvent(self, event):
-        '''Handle events and update dispatch table'''
+        """Handle events and update dispatch table."""
         XMLDirectoryBacked.HandleEvent(self, event)
         self.Entries = {}
         for src in self.entries.values():
@@ -476,7 +588,7 @@ class PrioDir(Plugin, Generator, XMLDirectoryBacked):
                         self.Entries[itype] = {child: self.BindEntry}
 
     def BindEntry(self, entry, metadata):
-        '''Check package lists of package entries'''
+        """Check package lists of package entries."""
         [src.Cache(metadata) for src in self.entries.values()]
         name = entry.get('name')
         if not src.cache:
@@ -512,7 +624,7 @@ class PrioDir(Plugin, Generator, XMLDirectoryBacked):
 # new unified EntrySet backend
 
 class SpecificityError(Exception):
-    '''Thrown in case of filename parse failure'''
+    """Thrown in case of filename parse failure."""
     pass
 
 class Specificity:
@@ -530,7 +642,7 @@ class Specificity:
                self.group in metadata.groups
 
     def __cmp__(self, other):
-        '''sort most to least specific'''
+        """Sort most to least specific."""
         if self.all:
             return 1
         if self.group:
@@ -543,7 +655,7 @@ class Specificity:
         return -1
 
     def more_specific(self, other):
-        '''test if self is more specific than other'''
+        """Test if self is more specific than other."""
         if self.all:
             True
         elif self.group:
@@ -567,7 +679,7 @@ class SpecificData(object):
             logger.error("Failed to read file %s" % self.name)
 
 class EntrySet:
-    '''Entry sets deal with the host- and group-specific entries'''
+    """Entry sets deal with the host- and group-specific entries."""
     ignore = re.compile("^(\.#.*|.*~|\\..*\\.(sw[px]))$")
     def __init__(self, basename, path, entry_type, encoding):
         self.path = path
@@ -585,7 +697,7 @@ class EntrySet:
                 if item.specific.matches(metadata)]
 
     def handle_event(self, event):
-        '''Handle FAM events for the TemplateSet'''
+        """Handle FAM events for the TemplateSet."""
         action = event.code2str()
 
         if event.filename in ['info', 'info.xml', ':info']:
@@ -606,7 +718,7 @@ class EntrySet:
                 del self.entries[event.filename]
 
     def entry_init(self, event):
-        '''handle template and info file creation'''
+        """Handle template and info file creation."""
         if event.filename in self.entries:
             logger.warn("Got duplicate add for %s" % event.filename)
         else:
@@ -622,7 +734,7 @@ class EntrySet:
         self.entries[event.filename].handle_event(event)
 
     def specificity_from_filename(self, fname):
-        '''construct a specificity instance from a filename and regex'''
+        """Construct a specificity instance from a filename and regex."""
         data = self.specific.match(fname)
         if not data:
             raise SpecificityError(fname)
@@ -639,7 +751,7 @@ class EntrySet:
         return Specificity(**kwargs)
 
     def update_metadata(self, event):
-        '''process info and info.xml files for the templates'''
+        """Process info and info.xml files for the templates."""
         fpath = "%s/%s" % (self.path, event.filename)
         if event.filename == 'info.xml':
             if not self.infoxml:
@@ -661,14 +773,14 @@ class EntrySet:
                                                  (self.metadata['perms'])
 
     def reset_metadata(self, event):
-        '''reset metadata to defaults if info or info.xml removed'''
+        """Reset metadata to defaults if info or info.xml removed."""
         if event.filename == 'info.xml':
             self.infoxml = None
         elif event.filename == 'info':
             self.metadata = default_file_metadata.copy()
 
     def group_sortfunc(self, x, y):
-        '''sort groups by their priority'''
+        """sort groups by their priority"""
         return cmp(x.specific.prio, y.specific.prio)
 
     def bind_info_to_entry(self, entry, metadata):
@@ -686,7 +798,7 @@ class EntrySet:
              for (key, value) in mdata['Info'][None].iteritems()]
 
     def bind_entry(self, entry, metadata):
-        '''Return the appropriate interpreted template from the set of available templates'''
+        """Return the appropriate interpreted template from the set of available templates."""
         self.bind_info_to_entry(entry, metadata)
         matching = self.get_matching(metadata)
 
@@ -706,9 +818,9 @@ class EntrySet:
         raise PluginExecutionError
 
 class GroupSpool(Plugin, Generator):
-    '''The TGenshi generator implements a templating mechanism for configuration files'''
+    """Unified interface for handling group-specific data (e.g. .G## files)."""
     name = 'GroupSpool'
-    __version__ = '$Id: Plugin.py 5651 2010-01-04 00:59:45Z solj $'
+    __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     filename_pattern = ""
     es_child_cls = object
@@ -719,7 +831,6 @@ class GroupSpool(Plugin, Generator):
         Generator.__init__(self)
         if self.data[-1] == '/':
             self.data = self.data[:-1]
-        self.Entries['ConfigFile'] = {}
         self.Entries['Path'] = {}
         self.entries = {}
         self.handles = {}
@@ -727,7 +838,7 @@ class GroupSpool(Plugin, Generator):
         self.encoding = core.encoding
 
     def HandleEvent(self, event):
-        '''Unified FAM event handler for DirShadow'''
+        """Unified FAM event handler for DirShadow."""
         action = event.code2str()
         if event.filename[0] == '/':
             return
@@ -747,7 +858,6 @@ class GroupSpool(Plugin, Generator):
                                                   dirpath,
                                                   self.es_child_cls,
                                                   self.encoding)
-                self.Entries['ConfigFile'][ident] = self.entries[ident].bind_entry
                 self.Entries['Path'][ident] = self.entries[ident].bind_entry
             if not posixpath.isdir(epath):
                 # do not pass through directory events
@@ -759,13 +869,12 @@ class GroupSpool(Plugin, Generator):
             if fbase in self.entries:
                 # a directory was deleted
                 del self.entries[fbase]
-                del self.Entries['ConfigFile'][fbase]
                 del self.Entries['Path'][fbase]
             else:
                 self.entries[ident].handle_event(event)
 
     def AddDirectoryMonitor(self, relative):
-        '''Add new directory to FAM structures'''
+        """Add new directory to FAM structures."""
         if not relative.endswith('/'):
             relative += '/'
         name = self.data + relative
