@@ -1,12 +1,12 @@
-'''This plugin provides automatic dependency handling'''
-__revision__ = '$Revision: 5477 $'
+"""This plugin provides automatic dependency handling."""
+__revision__ = '$Revision$'
 
 import lxml.etree
 
 import Bcfg2.Server.Plugin
 
 class DNode(Bcfg2.Server.Plugin.INode):
-    '''DNode provides supports for single predicate types for dependencies'''
+    """DNode provides supports for single predicate types for dependencies."""
     raw = {'Group':"lambda x:'%s' in x.groups and predicate(x)"}
     containers = ['Group']
 
@@ -39,7 +39,7 @@ class DepXMLSrc(Bcfg2.Server.Plugin.XMLSrc):
 class Deps(Bcfg2.Server.Plugin.PrioDir,
            Bcfg2.Server.Plugin.StructureValidator):
     name = 'Deps'
-    __version__ = '$Id: Deps.py 5477 2009-10-07 14:46:08Z solj $'
+    __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = DepXMLSrc
 
