@@ -1,9 +1,10 @@
 '''This module manages ssh key files for bcfg2'''
-__revision__ = '$Revision: 5477 $'
+__revision__ = '$Revision$'
 
 import binascii
 import os
 import socket
+import shutil
 import tempfile
 from subprocess import Popen, PIPE
 import Bcfg2.Server.Plugin
@@ -13,25 +14,27 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
               Bcfg2.Server.Plugin.Generator,
               Bcfg2.Server.Plugin.DirectoryBacked,
               Bcfg2.Server.Plugin.PullTarget):
-    '''The sshbase generator manages ssh host keys (both v1 and v2)
-    for hosts.  It also manages the ssh_known_hosts file. It can
-    integrate host keys from other management domains and similarly
-    export its keys. The repository contains files in the following
-    formats:
+    """
+       The sshbase generator manages ssh host keys (both v1 and v2)
+       for hosts.  It also manages the ssh_known_hosts file. It can
+       integrate host keys from other management domains and similarly
+       export its keys. The repository contains files in the following
+       formats:
 
-    ssh_host_key.H_(hostname) -> the v1 host private key for
-      (hostname)
-    ssh_host_key.pub.H_(hostname) -> the v1 host public key
-      for (hostname)
-    ssh_host_(dr)sa_key.H_(hostname) -> the v2 ssh host
-      private key for (hostname)
-    ssh_host_(dr)sa_key.pub.H_(hostname) -> the v2 ssh host
-      public key for (hostname)
-    ssh_known_hosts -> the current known hosts file. this
-      is regenerated each time a new key is generated.
-    '''
+       ssh_host_key.H_(hostname) -> the v1 host private key for
+         (hostname)
+       ssh_host_key.pub.H_(hostname) -> the v1 host public key
+         for (hostname)
+       ssh_host_(dr)sa_key.H_(hostname) -> the v2 ssh host
+         private key for (hostname)
+       ssh_host_(dr)sa_key.pub.H_(hostname) -> the v2 ssh host
+         public key for (hostname)
+       ssh_known_hosts -> the current known hosts file. this
+         is regenerated each time a new key is generated.
+
+    """
     name = 'SSHbase'
-    __version__ = '$Id: SSHbase.py 5477 2009-10-07 14:46:08Z solj $'
+    __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
 
     pubkeys = ["ssh_host_dsa_key.pub.H_%s",
@@ -54,7 +57,7 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                               % (self.data))
             self.logger.error(ioerr)
             raise Bcfg2.Server.Plugin.PluginInitError
-        self.Entries = {'ConfigFile':
+        self.Entries = {'Path':
                         {'/etc/ssh/ssh_known_hosts': self.build_skn,
                          '/etc/ssh/ssh_host_dsa_key': self.build_hk,
                          '/etc/ssh/ssh_host_rsa_key': self.build_hk,
@@ -63,10 +66,11 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                          '/etc/ssh/ssh_host_key': self.build_hk,
                          '/etc/ssh/ssh_host_key.pub': self.build_hk}}
         self.ipcache = {}
+        self.namecache = {}
         self.__skn = False
 
     def get_skn(self):
-        '''build memory cache of the ssh known hosts file'''
+        """Build memory cache of the ssh known hosts file."""
         if not self.__skn:
             self.__skn = "\n".join([value.data for key, value in \
                                     self.entries.iteritems() if \
@@ -80,14 +84,24 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                 names[cmeta.hostname] = set([cmeta.hostname])
                 names[cmeta.hostname].update(cmeta.aliases)
                 newnames = set()
+                newips = set()
                 for name in names[cmeta.hostname]:
                     newnames.add(name.split('.')[0])
                     try:
-                        newnames.add(self.get_ipcache_entry(name)[0])
+                        newips.add(self.get_ipcache_entry(name)[0])
                     except:
                         continue
                 names[cmeta.hostname].update(newnames)
                 names[cmeta.hostname].update(cmeta.addresses)
+                names[cmeta.hostname].update(newips)
+                # TODO: Only perform reverse lookups on IPs if an option is set.
+                if True:
+                    for ip in newips:
+                        try:
+                            names[cmeta.hostname].update(self.get_namecache_entry(ip))
+                        except:
+                            continue
+                names[cmeta.hostname] = sorted(names[cmeta.hostname])
             # now we have our name cache
             pubkeys = [pubk for pubk in self.entries.keys() \
                        if pubk.find('.pub.H_') != -1]
@@ -105,12 +119,12 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
         return self.__skn
 
     def set_skn(self, value):
-        '''Set backing data for skn'''
+        """Set backing data for skn."""
         self.__skn = value
     skn = property(get_skn, set_skn)
 
     def HandleEvent(self, event=None):
-        '''Local event handler that does skn regen on pubkey change'''
+        """Local event handler that does skn regen on pubkey change."""
         Bcfg2.Server.Plugin.DirectoryBacked.HandleEvent(self, event)
         if event and '_key.pub.H_' in event.filename:
             self.skn = False
@@ -121,21 +135,21 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                 _ = self.skn
 
     def HandlesEntry(self, entry, _):
-        '''Handle key entries dynamically'''
-        return entry.tag == 'ConfigFile' and \
+        """Handle key entries dynamically."""
+        return entry.tag == 'Path' and \
                ([fpat for fpat in self.keypatterns
                  if entry.get('name').endswith(fpat)]
                 or entry.get('name').endswith('ssh_known_hosts'))
 
     def HandleEntry(self, entry, metadata):
-        '''Bind data'''
+        """Bind data."""
         if entry.get('name').endswith('ssh_known_hosts'):
             return self.build_skn(entry, metadata)
         else:
             return self.build_hk(entry, metadata)
 
     def get_ipcache_entry(self, client):
-        '''build a cache of dns results'''
+        """Build a cache of dns results."""
         if client in self.ipcache:
             if self.ipcache[client]:
                 return self.ipcache[client]
@@ -158,8 +172,31 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                 self.logger.error("Failed to find IP address for %s" % client)
                 raise socket.gaierror
 
+    def get_namecache_entry(self, cip):
+        """Build a cache of name lookups from client IP addresses."""
+        if cip in self.namecache:
+            # lookup cached name from IP
+            if self.namecache[cip]:
+                return self.namecache[cip]
+            else:
+                raise socket.gaierror
+        else:
+            # add an entry that has not been cached
+            try:
+                rvlookup = socket.gethostbyaddr(cip)
+                if rvlookup[0]:
+                    self.namecache[cip] = [rvlookup[0]]
+                else:
+                    self.namecache[cip] = []
+                self.namecache[cip].extend(rvlookup[1])
+                return self.namecache[cip]
+            except socket.gaierror:
+                self.namecache[cip] = False
+                self.logger.error("Failed to find any names associated with IP address %s" % cip)
+                raise
+
     def build_skn(self, entry, metadata):
-        '''This function builds builds a host specific known_hosts file'''
+        """This function builds builds a host specific known_hosts file."""
         client = metadata.hostname
         entry.text = self.skn
         hostkeys = [keytmpl % client for keytmpl in self.pubkeys \
@@ -168,11 +205,14 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
         for hostkey in hostkeys:
             entry.text += "localhost,localhost.localdomain,127.0.0.1 %s" % (
                 self.entries[hostkey].data)
-        permdata = {'owner':'root', 'group':'root', 'perms':'0644'}
+        permdata = {'owner':'root',
+                    'group':'root',
+                    'type':'file',
+                    'perms':'0644'}
         [entry.attrib.__setitem__(key, permdata[key]) for key in permdata]
 
     def build_hk(self, entry, metadata):
-        '''This binds host key data into entries'''
+        """This binds host key data into entries."""
         client = metadata.hostname
         filename = "%s.H_%s" % (entry.get('name').split('/')[-1], client)
         if filename not in self.entries.keys():
@@ -181,8 +221,10 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
             self.logger.error("%s still not registered" % filename)
             raise Bcfg2.Server.Plugin.PluginExecutionError
         keydata = self.entries[filename].data
-        permdata = {'owner':'root', 'group':'root'}
-        permdata['perms'] = '0600'
+        permdata = {'owner':'root',
+                    'group':'root',
+                    'type':'file',
+                    'perms':'0600'}
         if entry.get('name')[-4:] == '.pub':
             permdata['perms'] = '0644'
         [entry.attrib.__setitem__(key, permdata[key]) for key in permdata]
@@ -193,7 +235,7 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
             entry.text = keydata
 
     def GenerateHostKeys(self, client):
-        '''Generate new host keys for client'''
+        """Generate new host keys for client."""
         keylist = [keytmpl % client for keytmpl in self.hostkeys]
         for hostkey in keylist:
             if 'ssh_host_rsa_key.H_' == hostkey[:19]:
@@ -212,8 +254,8 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
                 temploc = "%s/%s" % (tempdir, hostkey)
                 cmd = 'ssh-keygen -q -f %s -N "" -t %s -C root@%s < /dev/null'
                 os.system(cmd % (temploc, keytype, client))
-                open(fileloc, 'w').write(open(temploc).read())
-                open(publoc, 'w').write(open("%s.pub" % temploc).read())
+                shutil.copy(temploc, fileloc)
+                shutil.copy("%s.pub" % temploc, publoc)
                 self.AddEntry(hostkey)
                 self.AddEntry(".".join([hostkey.split('.')[0]]+['pub', "H_%s" \
                                                                 % client]))
@@ -228,7 +270,7 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,
         return [Bcfg2.Server.Plugin.Specificity(hostname=metadata.hostname)]
 
     def AcceptPullData(self, specific, entry, log):
-        '''per-plugin bcfg2-admin pull support'''
+        """Per-plugin bcfg2-admin pull support."""
         # specific will always be host specific
         filename = "%s/%s.H_%s" % (self.data, entry['name'].split('/')[-1],
                                    specific.hostname)
