@@ -1,5 +1,5 @@
 '''This file manages the statistics collected by the BCFG2 Server'''
-__revision__ = '$Revision: 5477 $'
+__revision__ = '$Revision$'
 
 import binascii
 import copy
@@ -8,13 +8,15 @@ import logging
 from lxml.etree import XML, SubElement, Element, XMLSyntaxError
 import lxml.etree
 import os
+import Queue
 from time import asctime, localtime, time, strptime, mktime
+import threading
 
 import Bcfg2.Server.Plugin
 
 
 class StatisticsStore(object):
-    '''Manages the memory and file copy of statistics collected about client runs'''
+    """Manages the memory and file copy of statistics collected about client runs."""
     __min_write_delay__ = 0
 
     def __init__(self, filename):
@@ -26,8 +28,7 @@ class StatisticsStore(object):
         self.ReadFromFile()
 
     def WriteBack(self, force=0):
-        '''Write statistics changes back to persistent store'''
-        # FIXME switch to a thread writer
+        """Write statistics changes back to persistent store."""
         if (self.dirty and (self.lastwrite + self.__min_write_delay__ <= time())) \
                 or force:
             try:
@@ -42,7 +43,7 @@ class StatisticsStore(object):
                 self.lastwrite = time()
 
     def ReadFromFile(self):
-        '''Reads current state regarding statistics'''
+        """Reads current state regarding statistics."""
         try:
             fin = open(self.filename, 'r')
             data = fin.read()
@@ -56,7 +57,7 @@ class StatisticsStore(object):
             self.dirty = 0
 
     def updateStats(self, xml, client):
-        '''Updates the statistics of a current node with new data'''
+        """Updates the statistics of a current node with new data."""
 
         # Current policy:
         # - Keep anything less than 24 hours old
@@ -97,14 +98,14 @@ class StatisticsStore(object):
         newstat.set('time', asctime(localtime()))
 
         # Add statistic
-        node.append(newstat)
+        node.append(copy.deepcopy(newstat))
 
         # Set dirty
         self.dirty = 1
         self.WriteBack(force=1)
 
     def isOlderThan24h(self, testTime):
-        '''Helper function to determine if <time> string is older than 24 hours'''
+        """Helper function to determine if <time> string is older than 24 hours."""
         now = time()
         utime = mktime(strptime(testTime))
         secondsPerDay = 60*60*24
@@ -113,20 +114,20 @@ class StatisticsStore(object):
 
 
 class Statistics(Bcfg2.Server.Plugin.Plugin,
-                 Bcfg2.Server.Plugin.Statistics,
+                 Bcfg2.Server.Plugin.ThreadedStatistics,
                  Bcfg2.Server.Plugin.PullSource):
     name = 'Statistics'
-    __version__ = '$Id: Statistics.py 5477 2009-10-07 14:46:08Z solj $'
+    __version__ = '$Id$'
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
-        Bcfg2.Server.Plugin.Statistics.__init__(self)
+        Bcfg2.Server.Plugin.ThreadedStatistics.__init__(self, core, datastore)
         Bcfg2.Server.Plugin.PullSource.__init__(self)
         fpath = "%s/etc/statistics.xml" % datastore
         self.data_file = StatisticsStore(fpath)
 
-    def process_statistics(self, client, xdata):
-        self.data_file.updateStats(copy.deepcopy(xdata), client.hostname)
+    def handle_statistic(self, metadata, data):
+        self.data_file.updateStats(data, metadata.hostname)
 
     def FindCurrent(self, client):
         rt = self.data_file.element.xpath('//Node[@name="%s"]' % client)[0]
